@@ -1,5 +1,5 @@
-use std::env::ArgsOs;
-use std::ffi::CString;
+// TODO: This file requires refactoring, but first I want to make sure the behavior is expected through testing.
+use std::ffi::{CString, OsString};
 use std::io::{self, Stderr, Stdout, Write};
 use std::os::raw::c_char;
 use std::process::exit;
@@ -11,14 +11,15 @@ use crossterm::{
     event::{self, Event, KeyCode, KeyEvent, KeyModifiers},
     execute, queue,
     style::{self, Attribute},
-    terminal::{self, ClearType},
+    terminal::{Clear, ClearType, EnterAlternateScreen, LeaveAlternateScreen},
 };
 
-use crate::args::{Parser, HELP};
+use crate::args::{ParsedArgs, Parser, StatusLine, HELP};
 use crate::error::Result;
 use crate::finder::Finder;
 use crate::matched_path::MatchedPath;
 use crate::starting_point::StartingPoint;
+use crate::terminal::Terminal;
 use crate::Error;
 
 pub fn safe_exit(code: i32, stdout: Stdout, stderr: Stderr) {
@@ -27,7 +28,11 @@ pub fn safe_exit(code: i32, stdout: Stdout, stderr: Stderr) {
     exit(code)
 }
 
-pub fn entrypoint(args: ArgsOs, stdout: &mut impl Write) -> Result<()> {
+pub fn entrypoint<A: Iterator<Item = OsString>, W: Write>(
+    args: A,
+    stdout: &mut W,
+    terminal: impl Terminal,
+) -> Result<()> {
     let args = Parser::new(args).parse()?;
     if args.help {
         print_and_flush(stdout, HELP)?;
@@ -41,25 +46,25 @@ pub fn entrypoint(args: ArgsOs, stdout: &mut impl Write) -> Result<()> {
         return Ok(());
     }
 
-    let (mut columns, mut rows) = terminal::size()?;
-    let starting_point = StartingPoint::new(args.starting_point)?;
-    let mut query = args.query;
-    let mut paths = find_paths(&starting_point, &query, paths_rows(rows))?;
+    let (mut columns, mut rows) = terminal.size()?;
+    let starting_point = StartingPoint::new(&args.starting_point)?;
+    let mut query = args.query.clone();
+    let mut paths = find_paths(&starting_point, &query, paths_rows(&args, rows))?;
     let mut selection: u16 = 0;
     let mut state = State::QueryChanged;
-    initialize_terminal(stdout)?;
+    initialize_terminal(stdout, &terminal)?;
 
     loop {
         match state {
             State::QueryChanged => {
-                output_on_terminal(stdout, &query, &paths[..], selection, columns, rows)?
+                output_on_terminal(stdout, &args, &query, &paths[..], selection, columns, rows)?
             }
             State::PathsChanged => {
-                output_on_terminal(stdout, &query, &paths[..], selection, columns, rows)?;
+                output_on_terminal(stdout, &args, &query, &paths[..], selection, columns, rows)?;
                 state = State::Ready;
             }
             State::SelectionChanged => {
-                output_on_terminal(stdout, &query, &paths[..], selection, columns, rows)?;
+                output_on_terminal(stdout, &args, &query, &paths[..], selection, columns, rows)?;
                 state = State::Ready;
             }
             _ => (),
@@ -85,34 +90,34 @@ pub fn entrypoint(args: ArgsOs, stdout: &mut impl Write) -> Result<()> {
                 }
                 state = State::SelectionChanged;
             } else if should_move_down(&ev) {
-                if selection < paths_rows(rows) - 1 {
+                if selection < paths_rows(&args, rows) - 1 {
                     selection += 1;
                 }
                 state = State::SelectionChanged;
             } else if ev == Event::Key(KeyCode::Enter.into()) {
                 let path: &MatchedPath = paths.get(selection as usize).unwrap(); // TODO: Do not use unwrap()
-                state = State::Invoke(&path);
+                state = State::Invoke(path);
                 break;
             } else if let Event::Resize(c, r) = ev {
                 columns = c;
                 rows = r;
                 selection = if selection > r {
-                    paths_rows(r) - 1
+                    paths_rows(&args, r) - 1
                 } else {
                     selection
                 };
-                paths = find_paths(&starting_point, &query, paths_rows(rows))?;
+                paths = find_paths(&starting_point, &query, paths_rows(&args, rows))?;
                 state = State::PathsChanged;
             }
         } else if let State::QueryChanged = state {
-            paths = find_paths(&starting_point, &query, paths_rows(rows))?;
+            paths = find_paths(&starting_point, &query, paths_rows(&args, rows))?;
             state = State::PathsChanged;
             selection = 0;
         }
     }
 
-    execute!(stdout, terminal::LeaveAlternateScreen)?;
-    terminal::disable_raw_mode()?;
+    execute!(stdout, LeaveAlternateScreen)?;
+    terminal.disable_raw_mode()?;
 
     if let State::Invoke(path) = state {
         invoke(&args.exec, path.absolute())?;
@@ -128,9 +133,12 @@ enum State<'a> {
     Invoke(&'a MatchedPath),
 }
 
-fn paths_rows(row: u16) -> u16 {
+fn paths_rows(args: &ParsedArgs, row: u16) -> u16 {
     // TODO: raise an error when the number of rows is too small.
-    row - 3
+    match args.status_line {
+        StatusLine::None => row - 2,
+        _ => row - 3,
+    }
 }
 
 fn should_just_exit(ev: &Event) -> bool {
@@ -165,15 +173,16 @@ fn print_and_flush(buffer: &mut impl Write, content: &str) -> io::Result<()> {
     buffer.flush()
 }
 
-fn initialize_terminal(stdout: &mut impl Write) -> Result<()> {
-    queue!(stdout, terminal::EnterAlternateScreen, style::ResetColor,)?;
+fn initialize_terminal(stdout: &mut impl Write, terminal: &impl Terminal) -> Result<()> {
+    queue!(stdout, EnterAlternateScreen, style::ResetColor)?;
     stdout.flush()?;
-    terminal::enable_raw_mode()?;
+    terminal.enable_raw_mode()?;
     Ok(())
 }
 
 fn output_on_terminal(
     stdout: &mut impl Write,
+    args: &ParsedArgs,
     query: &str,
     paths: &[MatchedPath],
     selection: u16,
@@ -183,7 +192,7 @@ fn output_on_terminal(
     queue!(
         stdout,
         cursor::MoveTo(0, 0),
-        terminal::Clear(ClearType::FromCursorDown),
+        Clear(ClearType::FromCursorDown),
         style::Print("Search: "),
         style::Print(query),
         cursor::SavePosition,
@@ -217,11 +226,18 @@ fn output_on_terminal(
         queue!(stdout, cursor::MoveToNextLine(1))?;
     }
 
-    queue!(stdout, cursor::MoveToRow(max_rows - 1))?;
-    let selected = &selected
-        .map(|s| s.truncated_absolute(max_columns as usize))
-        .unwrap_or_else(|| "No matching files found.".to_string());
-    status_line(stdout, max_columns as usize, selected)?;
+    let selected = match (selected, &args.status_line) {
+        (_, StatusLine::None) => None,
+        (None, _) => Some("No matching files found.".to_string()),
+        (Some(s), StatusLine::Absolute) => Some(s.truncated_absolute(max_columns as usize)),
+        (Some(s), StatusLine::Relative) => Some(s.truncated_relative(max_columns as usize)),
+    };
+    if let Some(ref s) = selected {
+        queue!(stdout, cursor::MoveToRow(max_rows - 1))?;
+        status_line(stdout, max_columns as usize, s)?;
+    } else {
+        queue!(stdout, cursor::MoveToRow(max_rows))?;
+    }
     help_line(stdout, max_columns as usize)?;
     queue!(stdout, cursor::RestorePosition)?;
     stdout.flush()?;
@@ -248,22 +264,22 @@ fn help_line(stdout: &mut impl Write, max_columns: usize) -> Result<()> {
         stdout,
         cursor::MoveToNextLine(1),
         style::SetAttribute(Attribute::Bold),
-        style::Print(format!("<Up>/<Ctrl-p>:")),
+        style::Print("<Up>/<Ctrl-p>:"),
         style::SetAttribute(Attribute::Reset),
         cursor::MoveRight(1),
-        style::Print(format!("Up")),
+        style::Print("Up"),
         cursor::MoveRight(2),
         style::SetAttribute(Attribute::Bold),
-        style::Print(format!("<Down>/<Ctrl-n>:")),
+        style::Print("<Down>/<Ctrl-n>:"),
         style::SetAttribute(Attribute::Reset),
         cursor::MoveRight(1),
-        style::Print(format!("Down")),
+        style::Print("Down"),
         cursor::MoveRight(2),
         style::SetAttribute(Attribute::Bold),
-        style::Print(format!("<Enter>:")),
+        style::Print("<Enter>:"),
         style::SetAttribute(Attribute::Reset),
         cursor::MoveRight(1),
-        style::Print(format!("Execute")),
+        style::Print("Execute"),
     )?;
     Ok(())
 }
