@@ -1,5 +1,7 @@
+pub use args::HELP;
 pub use cli::{entrypoint, safe_exit};
 pub use error::{Error, ErrorKind, Result};
+pub use terminal::{DefaultTerminal, Terminal};
 
 mod args;
 mod cli;
@@ -7,3 +9,4 @@ mod error;
 mod finder;
 mod matched_path;
 mod starting_point;
+mod terminal;
